Spell = {}
SPELL_WATER_LEVEL = 1500
SPELL_OPACITY = 0.65
SPELL_DURATION_HIDE_THRESHOLD = 20
DOT = "dot"
DEBUFF = "debuff"
BUFF = "buff"
PROTO_LINKABLE = "linkable"

function Spell:new(name, id, props, texture, timefont, stackfont)
	local object = { props = props, id = id, castTime = castTime, name = name, texture = texture, timefont = timefont,
					timeleft = 0, stacks = 0, shown = false, stackfont = stackfont, shader = nil, tex_verify = false,
					displayTimeleft = 0, disableStacks = false }
	object.shader = SpellShader:new(object);
	-- reset to yellow
	timefont:SetTextColor(1, 0.82, 0, 1);
	texture:SetDesaturated(nil);
	texture:SetAlpha(SPELL_OPACITY);
	setmetatable (object, { __index = Spell } )
	return object
end


-- timeleft in seconds
function Spell:updateDuration(timeleft)
	self.displayTimeleft = round(timeleft);
	timeleft = timeleft * 1000;
	
	--check for re-application
	if timeleft > self.timeleft then
		if self.props.spell_type == DOT then
			self:calculateTickTime();
		elseif self.props.spell_type == DEBUFF or self.props.spell_type == BUFF  then
			self:calculateDebuffThreshold();
		end 
	end
	self.timeleft = timeleft;
		
	if self.displayTimeleft < 0 then
		self.displayTimeleft = 0;
	elseif self.displayTimeleft < 5 then
		self.displayTimeleft = string.format("%1.1f", self.displayTimeleft);
	elseif self.displayTimeleft > SPELL_DURATION_HIDE_THRESHOLD then
		self.displayTimeleft = "";
	else
		self.displayTimeleft = string.format("%1d", self.displayTimeleft);
	end
	
	self.timefont:SetText(self.displayTimeleft);	
end

function Spell:calculateTickTime()
	local name, rank, icon, cost, isFunnel, powerType, castTime = GetSpellInfo(self.name);
	self.props.castTime = castTime;
	local playerHaste = UnitSpellHaste("player"); -- applies to ranged and melee too	
	self.props.tickTime = self.props.tick_frequency * 1000 / ( 1 + playerHaste / 100);
end

function Spell:calculateDebuffThreshold()
	if self.props.applied_by then
		local name, rank, icon, cost, isFunnel, powerType, castTime = GetSpellInfo(self.props.applied_by);
		self.props.castTime = castTime;		
	end
end

function round(val)
	return floor((val + 0.05)*10)/10
end

--override to use the timefont instead
function Spell:updateStacks(stacks)
	if (stacks > 0 or self.stacks > 0) and not self.disableStacks then
		self.stacks = stacks;
		self.timefont:SetText(self.stacks);	
		self.timefont:SetTextColor(1,0,0,1);
	end
end

function Spell:Show()
	if self.tex_verify and self.props.icon ~= self.texture:GetTexture() then		
		self.texture:SetTexture(self.props.icon);
	end
	self.shown = true;
	self.texture:Show();
	self.timefont:Show();
	if self.stacks > 0 then
		self.stackfont:Show();
	end
end

function Spell:Hide()
	self.shown = false;
	if not self.tex_verify or self.props.icon == self.texture:GetTexture() then	
		self.texture:Hide();
		self.timefont:Hide();
		self.stackfont:Hide();
	end
end

function Spell:IsShown()
	return self.shown;
end

function Spell:shade()	
	self.shader:doShade();		
end

function Spell:SetVerifyTexOnShow(value)
	self.tex_verify = value;
end